@@ -1,10 +1,15 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2012 Raphaël Barrois
+# Copyright (c) 2012-2013 Raphaël Barrois
+# This code is distributed under the two-clause BSD License.
 
+from __future__ import unicode_literals
 
 import functools
 import re
 
+
+from .compat import base_cmp
+
 def _to_int(value):
     try:
         return int(value), True
@@ -20,7 +25,7 @@ def identifier_cmp(a, b):
 
     if a_is_int and b_is_int:
         # Numeric identifiers are compared as integers
-        return cmp(a_cmp, b_cmp)
+        return base_cmp(a_cmp, b_cmp)
     elif a_is_int:
         # Numeric identifiers have lower precedence
         return -1
@@ -28,7 +33,7 @@ def identifier_cmp(a, b):
         return 1
     else:
         # Non-numeric identifers are compared lexicographically
-        return cmp(a_cmp, b_cmp)
+        return base_cmp(a_cmp, b_cmp)
 
 
 def identifier_list_cmp(a, b):
@@ -52,13 +57,13 @@ def identifier_list_cmp(a, b):
         if cmp_res != 0:
             return cmp_res
     # alpha1.3 < alpha1.3.1
-    return cmp(len(a), len(b))
+    return base_cmp(len(a), len(b))
 
 
 class Version(object):
 
     version_re = re.compile('^(\d+)\.(\d+)\.(\d+)(?:-([0-9a-zA-Z.-]+))?(?:\+([0-9a-zA-Z.-]+))?$')
-    partial_version_re = re.compile('^(\d+)\.(\d+)\.(\d+)(?:-([0-9a-zA-Z.-]*))?(?:\+([0-9a-zA-Z.-]*))?$')
+    partial_version_re = re.compile('^(\d+)(?:\.(\d+)(?:\.(\d+))?)?(?:-([0-9a-zA-Z.-]*))?(?:\+([0-9a-zA-Z.-]*))?$')
 
     def __init__(self, version_string, partial=False):
         major, minor, patch, prerelease, build = self.parse(version_string, partial)
@@ -72,7 +77,91 @@ class Version(object):
         self.partial = partial
 
     @classmethod
-    def parse(cls, version_string, partial=False):
+    def _coerce(cls, value, allow_none=False):
+        if value is None and allow_none:
+            return value
+        return int(value)
+
+    @classmethod
+    def coerce(cls, version_string, partial=False):
+        """Coerce an arbitrary version string into a semver-compatible one.
+
+        The rule is:
+        - If not enough components, fill minor/patch with zeroes; unless
+          partial=True
+        - If more than 3 dot-separated components, extra components are "build"
+          data. If some "build" data already appeared, append it to the
+          extra components
+
+        Examples:
+            >>> Version.coerce('0.1')
+            Version(0, 1, 0)
+            >>> Version.coerce('0.1.2.3')
+            Version(0, 1, 2, (), ('3',))
+            >>> Version.coerce('0.1.2.3+4')
+            Version(0, 1, 2, (), ('3', '4'))
+            >>> Version.coerce('0.1+2-3+4_5')
+            Version(0, 1, 0, (), ('2-3', '4-5'))
+        """
+        base_re = re.compile(r'^\d+(?:\.\d+(?:\.\d+)?)?')
+
+        match = base_re.match(version_string)
+        if not match:
+            raise ValueError("Version string lacks a numerical component: %r"
+                    % version_string)
+
+        version = version_string[:match.end()]
+        if not partial:
+            # We need a not-partial version.
+            while version.count('.') < 2:
+                version += '.0'
+
+        if match.end() == len(version_string):
+            return Version(version, partial=partial)
+
+        rest = version_string[match.end():]
+
+        # Cleanup the 'rest'
+        rest = re.sub(r'[^a-zA-Z0-9+.-]', '-', rest)
+
+        if rest[0] == '+':
+            # A 'build' component
+            prerelease = ''
+            build = rest[1:]
+        elif rest[0] == '.':
+            # An extra version component, probably 'build'
+            prerelease = ''
+            build = rest[1:]
+        elif rest[0] == '-':
+            rest = rest[1:]
+            if '+' in rest:
+                prerelease, build = rest.split('+', 1)
+            else:
+                prerelease, build = rest, ''
+        elif '+' in rest:
+            prerelease, build = rest.split('+', 1)
+        else:
+            prerelease, build = rest, ''
+
+        build = build.replace('+', '.')
+
+        if prerelease:
+            version = '%s-%s' % (version, prerelease)
+        if build:
+            version = '%s+%s' % (version, build)
+
+        return cls(version, partial=partial)
+
+    @classmethod
+    def parse(cls, version_string, partial=False, coerce=False):
+        """Parse a version string into a Version() object.
+
+        Args:
+            version_string (str), the version string to parse
+            partial (bool), whether to accept incomplete input
+            coerce (bool), whether to try to map the passed in string into a
+                valid Version.
+        """
         if not version_string:
             raise ValueError('Invalid empty version string: %r' % version_string)
 
@@ -88,8 +177,8 @@ class Version(object):
         major, minor, patch, prerelease, build = match.groups()
 
         major = int(major)
-        minor = int(minor)
-        patch = int(patch)
+        minor = cls._coerce(minor, partial)
+        patch = cls._coerce(patch, partial)
 
         if prerelease is None:
             if partial and (build is None):
@@ -118,7 +207,11 @@ class Version(object):
         return iter((self.major, self.minor, self.patch, self.prerelease, self.build))
 
     def __str__(self):
-        version = '%d.%d.%d' % (self.major, self.minor, self.patch)
+        version = '%d' % self.major
+        if self.minor is not None:
+            version = '%s.%d' % (version, self.minor)
+        if self.patch is not None:
+            version = '%s.%d' % (version, self.patch)
 
         if self.prerelease or (self.partial and self.prerelease == () and self.build is None):
             version = '%s-%s' % (version, '.'.join(self.prerelease))
@@ -132,9 +225,6 @@ class Version(object):
             ', partial=True' if self.partial else '',
         )
 
-    def __hash__(self):
-        return hash((self.major, self.minor, self.patch, self.prerelease, self.build))
-
     @classmethod
     def _comparison_functions(cls, partial=False):
         """Retrieve comparison methods to apply on version components.
@@ -192,17 +282,17 @@ class Version(object):
 
         if partial:
             return [
-                cmp,  # Major is still mandatory
-                make_optional(cmp),
-                make_optional(cmp),
+                base_cmp,  # Major is still mandatory
+                make_optional(base_cmp),
+                make_optional(base_cmp),
                 make_optional(prerelease_cmp),
                 make_optional(build_cmp),
             ]
         else:
             return [
-                cmp,
-                cmp,
-                cmp,
+                base_cmp,
+                base_cmp,
+                base_cmp,
                 prerelease_cmp,
                 build_cmp,
             ]
@@ -213,15 +303,54 @@ class Version(object):
 
         field_pairs = zip(self, other)
         comparison_functions = self._comparison_functions(partial=self.partial or other.partial)
+        comparisons = zip(comparison_functions, self, other)
 
-        for cmp_fun, field_pair in zip(comparison_functions, field_pairs):
-            self_field, other_field = field_pair
+        for cmp_fun, self_field, other_field in comparisons:
             cmp_res = cmp_fun(self_field, other_field)
             if cmp_res != 0:
                 return cmp_res
 
         return 0
 
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+
+        return self.__cmp__(other) == 0
+
+    def __hash__(self):
+        return hash((self.major, self.minor, self.patch, self.prerelease, self.build))
+
+    def __ne__(self, other):
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+
+        return self.__cmp__(other) != 0
+
+    def __lt__(self, other):
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+
+        return self.__cmp__(other) < 0
+
+    def __le__(self, other):
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+
+        return self.__cmp__(other) <= 0
+
+    def __gt__(self, other):
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+
+        return self.__cmp__(other) > 0
+
+    def __ge__(self, other):
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+
+        return self.__cmp__(other) >= 0
+
 
 class SpecItem(object):
     """A requirement specification."""
@@ -345,8 +474,17 @@ class Spec(object):
 
 
 def compare(v1, v2):
-    return cmp(Version(v1), Version(v2))
+    return base_cmp(Version(v1), Version(v2))
 
 
 def match(spec, version):
     return Spec(spec).match(Version(version))
+
+
+def validate(version_string):
+    """Validates a version string againt the SemVer specification."""
+    try:
+        Version.parse(version_string)
+        return True
+    except ValueError:
+        return False
