@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2012 Raphaël Barrois
+# Copyright (c) 2012-2013 Raphaël Barrois
+# This code is distributed under the two-clause BSD License.
+
+from __future__ import unicode_literals
 
 from django.db import models
 from django.utils.translation import ugettext_lazy as _
@@ -26,15 +29,24 @@ class BaseSemVerField(models.CharField):
         value = self.to_python(self._get_val_from_obj(obj))
         return str(value)
 
+    def run_validators(self, value):
+        return super(BaseSemVerField, self).run_validators(str(value))
 
-class VersionField(BaseSemVerField):
+
+# Py2 and Py3-compatible metaclass
+SemVerField = models.SubfieldBase(
+    str('SemVerField'), (BaseSemVerField, models.CharField), {})
+
+
+class VersionField(SemVerField):
     default_error_messages = {
-        'invalid': _(u"Enter a valid version number in X.Y.Z format."),
+        'invalid': _("Enter a valid version number in X.Y.Z format."),
     }
-    description = _(u"Version")
+    description = _("Version")
 
     def __init__(self, *args, **kwargs):
         self.partial = kwargs.pop('partial', False)
+        self.coerce = kwargs.pop('coerce', False)
         super(VersionField, self).__init__(*args, **kwargs)
 
     def to_python(self, value):
@@ -43,14 +55,17 @@ class VersionField(BaseSemVerField):
             return value
         if isinstance(value, base.Version):
             return value
-        return base.Version(value, partial=self.partial)
+        if self.coerce:
+            return base.Version.coerce(value, partial=self.partial)
+        else:
+            return base.Version(value, partial=self.partial)
 
 
-class SpecField(BaseSemVerField):
+class SpecField(SemVerField):
     default_error_messages = {
-        'invalid': _(u"Enter a valid version number spec list in ==X.Y.Z,>=A.B.C format."),
+        'invalid': _("Enter a valid version number spec list in ==X.Y.Z,>=A.B.C format."),
     }
-    description = _(u"Version specification list")
+    description = _("Version specification list")
 
     def to_python(self, value):
         """Converts any value to a base.Spec field."""
@@ -68,7 +83,10 @@ def add_south_rules():
         (
             (VersionField,),
             [],
-            {'partial': ('partial', {'default': False})},
+            {
+                'partial': ('partial', {'default': False}),
+                'coerce': ('coerce', {'default': False}),
+            },
         ),
     ], ["semantic_version\.django_fields"])
 
