@@ -1,14 +1,11 @@
 #!/usr/bin/env python
-# coding: utf-8
-# Copyright (c) 2012 Raphaël Barrois
+# -*- coding: utf-8 -*-
+# Copyright (c) 2012-2013 Raphaël Barrois
+# This code is distributed under the two-clause BSD License.
 
 """Test the various functions from 'base'."""
 
-try:  # pragma: no cover
-    import unittest2 as unittest
-except ImportError:  # pragma: no cover
-    import unittest
-
+from .compat import unittest, is_python2
 
 from semantic_version import base
 
@@ -88,6 +85,46 @@ class TopLevelTestCase(unittest.TestCase):
             self.assertTrue(base.match(spec, version),
                 "%r should accept %r" % (spec, version))
 
+    valid_strings = (
+        '1.0.0-alpha',
+        '1.0.0-alpha.1',
+        '1.0.0-beta.2',
+        '1.0.0-beta.11',
+        '1.0.0-rc.1',
+        '1.0.0-rc.1+build.1',
+        '1.0.0',
+        '1.0.0+0.3.7',
+        '1.3.7+build',
+        '1.3.7+build.2.b8f12d7',
+        '1.3.7+build.11.e0f985a',
+        '1.1.1',
+        '1.1.2',
+        '1.1.3-rc4.5',
+        '1.1.3-rc42.3-14-15.24+build.2012-04-13.223',
+        '1.1.3+build.2012-04-13.HUY.alpha-12.1',
+    )
+
+    def test_validate_valid(self):
+        for version in self.valid_strings:
+            self.assertTrue(base.validate(version),
+                "%r should be a valid version" % (version,))
+
+    invalid_strings = (
+        '1',
+        'v1',
+        '1.2.3.4',
+        '1.2',
+        '1.2a3',
+        '1.2.3a4',
+        'v12.34.5',
+        '1.2.3+4+5',
+    )
+
+    def test_validate_invalid(self):
+        for version in self.invalid_strings:
+            self.assertFalse(base.validate(version),
+                "%r should not be a valid version" % (version,))
+
 
 class VersionTestCase(unittest.TestCase):
     versions = {
@@ -130,6 +167,8 @@ class VersionTestCase(unittest.TestCase):
             self.assertNotEqual(text, base.Version(text))
 
     partial_versions = {
+        '1.1': (1, 1, None, None, None),
+        '2': (2, None, None, None, None),
         '1.0.0-alpha': (1, 0, 0, ('alpha',), None),
         '1.0.0-alpha.1': (1, 0, 0, ('alpha', '1'), None),
         '1.0.0-beta.2': (1, 0, 0, ('beta', '2'), None),
@@ -188,6 +227,21 @@ class VersionTestCase(unittest.TestCase):
             ]))
         )
 
+    @unittest.skipIf(is_python2, "Comparisons to other objects are broken in Py2.")
+    def test_invalid_comparisons(self):
+        v = base.Version('0.1.0')
+        with self.assertRaises(TypeError):
+            v < '0.1.0'
+        with self.assertRaises(TypeError):
+            v <= '0.1.0'
+        with self.assertRaises(TypeError):
+            v > '0.1.0'
+        with self.assertRaises(TypeError):
+            v >= '0.1.0'
+
+        self.assertTrue(v != '0.1.0')
+        self.assertFalse(v == '0.1.0')
+
 
 class SpecItemTestCase(unittest.TestCase):
     components = {
@@ -197,6 +251,7 @@ class SpecItemTestCase(unittest.TestCase):
         '<=0.1.1+': (base.SpecItem.KIND_LTE, 0, 1, 1, (), ()),
         '<0.1.1': (base.SpecItem.KIND_LT, 0, 1, 1, None, None),
         '<=0.1.1': (base.SpecItem.KIND_LTE, 0, 1, 1, None, None),
+        '<=0.1.1-': (base.SpecItem.KIND_LTE, 0, 1, 1, (), None),
         '>=0.2.3-rc2': (base.SpecItem.KIND_GTE, 0, 2, 3, ('rc2',), None),
         '>0.2.3-rc2+': (base.SpecItem.KIND_GT, 0, 2, 3, ('rc2',), ()),
         '>=2.0.0': (base.SpecItem.KIND_GTE, 2, 0, 0, None, None),
@@ -300,6 +355,29 @@ class SpecItemTestCase(unittest.TestCase):
             len(set([base.SpecItem('==0.1.0'), base.SpecItem('==0.1.0')])))
 
 
+class CoerceTestCase(unittest.TestCase):
+    examples = {
+        # Dict of target: [list of equivalents]
+        '0.0.0': ('0', '0.0', '0.0.0', '0.0.0+', '0-', '00000000.00'),
+        '0.1.0': ('0.1', '0.1+', '0.1-', '0.1.0', '0.000001.000000000000'),
+        '0.1.0+2': ('0.1.0+2', '0.1.0.2'),
+        '0.1.0+2.3.4': ('0.1.0+2.3.4', '0.1.0+2+3+4', '0.1.0.2+3+4'),
+        '0.1.0+2-3.4': ('0.1.0+2-3.4', '0.1.0+2-3+4', '0.1.0.2-3+4', '0.1.0.2_3+4'),
+        '0.1.0-a2.3': ('0.1.0-a2.3', '0.1.0a2.3', '0.1.0_a2.3'),
+        '0.1.0-a2.3+4.5-6': ('0.1.0-a2.3+4.5-6', '0.1.0a2.3+4.5-6', '0.1.0a2.3+4.5_6', '0.1.0a2.3+4+5/6'),
+    }
+
+    def test_coerce(self):
+        for equivalent, samples in self.examples.items():
+            target = base.Version(equivalent)
+            for sample in samples:
+                v_sample = base.Version.coerce(sample)
+                self.assertEqual(target, v_sample)
+
+    def test_invalid(self):
+        self.assertRaises(ValueError, base.Version.coerce, 'v1')
+
+
 class SpecTestCase(unittest.TestCase):
     examples = {
         '>=0.1.1,<0.1.2': ['>=0.1.1', '<0.1.2'],
