@@ -1,6 +1,8 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2012 Raphaël Barrois
+# Copyright (c) 2012-2013 Raphaël Barrois
+# This code is distributed under the two-clause BSD License.
+
+from __future__ import unicode_literals
 
 try:  # pragma: no cover
     import unittest2 as unittest
@@ -28,7 +30,7 @@ if django_loaded:  # pragma: no cover
                 'tests.django_test_app',
             ]
         )
-    from django_test_app import models
+    from .django_test_app import models
     from django.core import serializers
 
 try:  # pragma: no cover
@@ -54,13 +56,27 @@ class DjangoFieldTestCase(unittest.TestCase):
         self.assertEqual(obj.spec, alt_obj.spec)
         self.assertEqual(obj.version, alt_obj.version)
 
+        obj.full_clean()
+
+    def test_partial_spec(self):
+        obj = models.VersionModel(version='0.1.1', spec='==0,!=0.2')
+        self.assertEqual(semantic_version.Version('0.1.1'), obj.version)
+        self.assertEqual(semantic_version.Spec('==0,!=0.2'), obj.spec)
+
+    def test_coerce(self):
+        obj = models.CoerceVersionModel(version='0.1.1a+2', partial='23')
+        self.assertEqual(semantic_version.Version('0.1.1-a+2'), obj.version)
+        self.assertEqual(semantic_version.Version('23', partial=True), obj.partial)
+
+        obj2 = models.CoerceVersionModel(version='23', partial='0.1.2.3.4.5/6')
+        self.assertEqual(semantic_version.Version('23.0.0'), obj2.version)
+        self.assertEqual(semantic_version.Version('0.1.2+3.4.5-6', partial=True), obj2.partial)
+
     def test_invalid_input(self):
         self.assertRaises(ValueError, models.VersionModel,
             version='0.1.1', spec='blah')
         self.assertRaises(ValueError, models.VersionModel,
             version='0.1', spec='==0.1.1,!=0.1.1-alpha')
-        self.assertRaises(ValueError, models.VersionModel,
-            version='0.1.1', spec='==0,!=0.2')
 
     def test_partial(self):
         obj = models.PartialVersionModel(partial='0.1.0')
@@ -81,6 +97,8 @@ class DjangoFieldTestCase(unittest.TestCase):
         self.assertIsNone(obj.optional)
         self.assertIsNone(obj.optional_spec)
 
+        obj.full_clean()
+
     def test_serialization(self):
         o1 = models.VersionModel(version='0.1.1', spec='==0.1.1,!=0.1.1-alpha')
         o2 = models.VersionModel(version='0.4.3-rc3+build3',
@@ -128,6 +146,15 @@ class SouthTestCase(unittest.TestCase):
         self.assertEqual(frozen['optional_spec'],
             ('semantic_version.django_fields.SpecField', [], {'max_length': '200', 'blank': 'True', 'null': 'True'}))
 
+    def test_freezing_coerce_version_model(self):
+        frozen = south.modelsinspector.get_model_fields(models.CoerceVersionModel)
+
+        self.assertEqual(frozen['version'],
+            ('semantic_version.django_fields.VersionField', [], {'max_length': '200', 'coerce': 'True'}))
+
+        self.assertEqual(frozen['partial'],
+                ('semantic_version.django_fields.VersionField', [], {'max_length': '200', 'partial': 'True', 'coerce': 'True'}))
+
     def test_freezing_app(self):
         frozen = south.creator.freezer.freeze_apps('django_test_app')
 
@@ -148,6 +175,13 @@ class SouthTestCase(unittest.TestCase):
         self.assertEqual(frozen['django_test_app.partialversionmodel']['optional_spec'],
             ('semantic_version.django_fields.SpecField', [], {'max_length': '200', 'blank': 'True', 'null': 'True'}))
 
+        # Test CoerceVersionModel
+        self.assertEqual(frozen['django_test_app.coerceversionmodel']['version'],
+            ('semantic_version.django_fields.VersionField', [], {'max_length': '200', 'coerce': 'True'}))
+
+        self.assertEqual(frozen['django_test_app.coerceversionmodel']['partial'],
+            ('semantic_version.django_fields.VersionField', [], {'max_length': '200', 'partial': 'True', 'coerce': 'True'}))
+
 
 if django_loaded:
     from django.test import TestCase
